@@ -23,6 +23,7 @@ from localstack.constants import (
     LOCALHOST,
     LOCALHOST_IP,
     LOG_LEVELS,
+    TRACE_LOG_LEVELS,
     TRUE_STRINGS,
 )
 
@@ -150,7 +151,7 @@ HOST_TMP_FOLDER = os.environ.get("HOST_TMP_FOLDER", TMP_FOLDER)
 
 # whether to enable verbose debug logging
 LS_LOG = eval_log_type("LS_LOG")
-DEBUG = is_env_true("DEBUG") or LS_LOG == "trace"
+DEBUG = is_env_true("DEBUG") or LS_LOG in TRACE_LOG_LEVELS
 
 # whether to enable debugpy
 DEVELOP = is_env_true("DEVELOP")
@@ -275,6 +276,10 @@ LAMBDA_FALLBACK_URL = os.environ.get("LAMBDA_FALLBACK_URL", "").strip()
 # Forward URL used to forward any Lambda invocations to an external
 # endpoint (can use useful for advanced test setups)
 LAMBDA_FORWARD_URL = os.environ.get("LAMBDA_FORWARD_URL", "").strip()
+# Time in seconds to wait at max while extracting Lambda code.
+# By default it is 25 seconds for limiting the execution time
+# to avoid client/network timeout issues
+LAMBDA_CODE_EXTRACT_TIME = int(os.environ.get("LAMBDA_CODE_EXTRACT_TIME") or 25)
 
 # A comma-delimited string of stream names and its corresponding shard count to
 # initialize during startup.
@@ -335,6 +340,7 @@ CONFIG_ENV_VARS = [
     "TF_COMPAT_MODE",
     "LAMBDA_DOCKER_FLAGS",
     "LAMBDA_FORWARD_URL",
+    "LAMBDA_CODE_EXTRACT_TIME",
     "THUNDRA_APIKEY",
     "THUNDRA_AGENT_JAVA_VERSION",
 ]
@@ -451,7 +457,11 @@ else:
 CLI_COMMANDS = {}
 
 # set of valid regions
-VALID_REGIONS = set(Session().get_available_regions("sns"))
+VALID_PARTITIONS = set(Session().get_available_partitions())
+VALID_REGIONS = set()
+for partition in VALID_PARTITIONS:
+    for region in Session().get_available_regions("sns", partition):
+        VALID_REGIONS.add(region)
 
 
 def parse_service_ports():
@@ -527,7 +537,7 @@ def service_port(service_key):
             #  the edge service, as that would require too many route mappings. In the future, we
             #  should integrate them with the port range for external services (4510-4530)
             return SERVICE_PORTS.get(service_key, 0)
-        return EDGE_PORT_HTTP or EDGE_PORT
+        return get_edge_port_http()
     return SERVICE_PORTS.get(service_key, 0)
 
 
@@ -540,9 +550,15 @@ def external_service_url(service_key, host=None):
     return "%s://%s:%s" % (get_protocol(), host, service_port(service_key))
 
 
-def get_edge_url():
-    port = EDGE_PORT_HTTP or EDGE_PORT
-    return "%s://%s:%s" % (get_protocol(), LOCALSTACK_HOSTNAME, port)
+def get_edge_port_http():
+    return EDGE_PORT_HTTP or EDGE_PORT
+
+
+def get_edge_url(localstack_hostname=None, protocol=None):
+    port = get_edge_port_http()
+    protocol = protocol or get_protocol()
+    localstack_hostname = localstack_hostname or LOCALSTACK_HOSTNAME
+    return "%s://%s:%s" % (protocol, localstack_hostname, port)
 
 
 # initialize config values
@@ -570,7 +586,7 @@ def load_config_file(config_file=None):
     return configs
 
 
-if LS_LOG == "trace":
+if LS_LOG in TRACE_LOG_LEVELS:
     load_end_time = time.time()
     LOG = logging.getLogger(__name__)
     LOG.debug(
