@@ -4,7 +4,7 @@ import json
 import logging
 import re
 import time
-from typing import Dict, Union
+from typing import Dict, Tuple, Union
 
 import requests
 from flask import Response as FlaskResponse
@@ -12,9 +12,10 @@ from moto.apigateway.models import apigateway_backends
 from requests.models import Response
 from six.moves.urllib_parse import urljoin
 
-from localstack.config import TEST_KINESIS_URL, TEST_SQS_URL
+from localstack import config
 from localstack.constants import (
     APPLICATION_JSON,
+    HEADER_LOCALSTACK_EDGE_URL,
     LOCALHOST_HOSTNAME,
     PATH_USER_REQUEST,
     TEST_AWS_ACCOUNT_ID,
@@ -56,19 +57,24 @@ from localstack.utils.aws.aws_responses import (
     requests_response,
 )
 from localstack.utils.aws.request_context import MARKER_APIGW_REQUEST_REGION, THREAD_LOCAL
-from localstack.utils.common import json_safe, to_bytes, to_str
+from localstack.utils.common import camel_to_snake_case, json_safe, to_bytes, to_str
 
 # set up logger
 LOG = logging.getLogger(__name__)
 
-# regex path patterns
-HOST_REGEX_EXECUTE_API = r"(?:.*://)?([a-zA-Z0-9-]+)\.execute-api\..*"
+# target ARN patterns
 TARGET_REGEX_S3_URI = (
     r"^arn:aws:apigateway:[a-zA-Z0-9\-]+:s3:path/(?P<bucket>[^/]+)/(?P<object>.+)$"
 )
+# regex path pattern for user requests
 PATH_REGEX_USER_REQUEST = (
     r"^/restapis/([A-Za-z0-9_\-]+)/([A-Za-z0-9_\-]+)/%s/(.*)$" % PATH_USER_REQUEST
 )
+# URL pattern for invocations
+HOST_REGEX_EXECUTE_API = (
+    r"(?:.*://)?([a-zA-Z0-9-]+)\.execute-api\.(%s|([^\.]+)\.amazonaws\.com)(.*)"
+    % LOCALHOST_HOSTNAME
+)
 
 
 class AuthorizationError(Exception):
@@ -77,7 +83,9 @@ class AuthorizationError(Exception):
 
 class ProxyListenerApiGateway(ProxyListener):
     def forward_request(self, method, path, data, headers):
-        if re.match(PATH_REGEX_USER_REQUEST, path):
+
+        forwarded_for = headers.get(HEADER_LOCALSTACK_EDGE_URL, "")
+        if re.match(PATH_REGEX_USER_REQUEST, path) or "execute-api" in forwarded_for:
             result = invoke_rest_api_from_request(method, path, data, headers)
             if result is not None:
                 return result
@@ -255,18 +263,20 @@ def apply_response_parameters(response, integration, api_id=None):
     return response
 
 
-def get_api_id_stage_invocation_path(path, headers):
+def get_api_id_stage_invocation_path(path: str, headers: Dict[str, str]) -> Tuple[str, str, str]:
     path_match = re.search(PATH_REGEX_USER_REQUEST, path)
-    host_header = headers.get("Host", "")
+    host_header = headers.get(HEADER_LOCALSTACK_EDGE_URL, "") or headers.get("Host") or ""
     host_match = re.search(HOST_REGEX_EXECUTE_API, host_header)
     if path_match:
         api_id = path_match.group(1)
         stage = path_match.group(2)
         relative_path_w_query_params = "/%s" % path_match.group(3)
     elif host_match:
-        api_id = host_match.group(1)
+        api_id = extract_api_id_from_hostname_in_url(host_header)
         stage = path.strip("/").split("/")[0]
         relative_path_w_query_params = "/%s" % path.lstrip("/").partition("/")[2]
+    else:
+        raise Exception(f"Unable to extract API Gateway details from request: {path} {headers}")
     if api_id:
         # set current region in request thread local, to ensure aws_stack.get_region() works properly
         if getattr(THREAD_LOCAL, "request_context", None) is not None:
@@ -276,6 +286,13 @@ def get_api_id_stage_invocation_path(path, headers):
     return api_id, stage, relative_path_w_query_params
 
 
+def extract_api_id_from_hostname_in_url(hostname: str) -> str:
+    """Extract API ID 'id123' from URLs like https://id123.execute-api.localhost.localstack.cloud:4566"""
+    match = re.match(HOST_REGEX_EXECUTE_API, hostname)
+    api_id = match.group(1)
+    return api_id
+
+
 def invoke_rest_api_from_request(method, path, data, headers, context={}, auth_info={}, **kwargs):
     api_id, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(path, headers)
     try:
@@ -527,7 +544,7 @@ def invoke_rest_api_integration_backend(
             headers = aws_stack.mock_aws_request_headers(service="kinesis")
             headers["X-Amz-Target"] = target
             result = common.make_http_request(
-                url=TEST_KINESIS_URL, method="POST", data=new_data, headers=headers
+                url=config.TEST_KINESIS_URL, method="POST", data=new_data, headers=headers
             )
             # apply response template
             result = apply_request_response_templates(
@@ -537,38 +554,40 @@ def invoke_rest_api_integration_backend(
 
         elif "states:action/" in uri:
             action = uri.split("/")[-1]
-            decoded_data = data.decode()
-            if "stateMachineArn" in decoded_data and "input" in decoded_data:
-                payload = json.loads(decoded_data)
-            else:
-                # apply request templates
+            payload = {}
+
+            if APPLICATION_JSON in integration.get("requestTemplates", {}):
                 payload = apply_request_response_templates(
                     data,
                     integration.get("requestTemplates"),
                     content_type=APPLICATION_JSON,
                     as_json=True,
                 )
-
+            else:
+                payload = json.loads(data.decode("utf-8"))
             client = aws_stack.connect_to_service("stepfunctions")
 
-            kwargs = {"name": payload["name"]} if "name" in payload else {}
-            sm_arn = payload.get("stateMachineArn")
-            if not sm_arn:
-                return make_error_response(
-                    "Missing 'stateMachineArn' in API GW request payload",
-                    400,
-                )
-            result = client.start_execution(
-                stateMachineArn=sm_arn,
-                input=payload.get("input") or "{}",
-                **kwargs,
+            # Hot fix since step functions local package responses: Unsupported Operation: 'StartSyncExecution'
+            method_name = (
+                camel_to_snake_case(action) if action != "StartSyncExecution" else "start_execution"
             )
+
+            try:
+                method = getattr(client, method_name)
+            except AttributeError:
+                msg = "Invalid step function action: %s" % method_name
+                LOG.error(msg)
+                return make_error_response(msg, 400)
+
+            result = method(
+                **payload,
+            )
+            result = json_safe({k: result[k] for k in result if k not in "ResponseMetadata"})
             response = requests_response(
-                content={
-                    "executionArn": result["executionArn"],
-                    "startDate": str(result["startDate"]),
-                },
+                content=result,
+                headers=aws_stack.mock_aws_request_headers(),
             )
+
             if action == "StartSyncExecution":
                 # poll for the execution result and return it
                 result = await_sfn_execution_result(result["executionArn"])
@@ -625,7 +644,7 @@ def invoke_rest_api_integration_backend(
                 )
                 headers = aws_stack.mock_aws_request_headers(service="sqs", region_name=region_name)
 
-                url = urljoin(TEST_SQS_URL, "%s/%s" % (TEST_AWS_ACCOUNT_ID, queue))
+                url = urljoin(config.TEST_SQS_URL, "%s/%s" % (TEST_AWS_ACCOUNT_ID, queue))
                 result = common.make_http_request(
                     url, method="POST", headers=headers, data=new_request
                 )
