@@ -18,7 +18,15 @@ from localstack.services.generic_proxy import ProxyListener, RegionBackend
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_responses, aws_stack
 from localstack.utils.bootstrap import is_api_enabled
-from localstack.utils.common import clone, json_safe, select_attributes, short_uid, to_bytes, to_str
+from localstack.utils.common import (
+    clone,
+    json_safe,
+    long_uid,
+    select_attributes,
+    short_uid,
+    to_bytes,
+    to_str,
+)
 
 # set up logger
 LOGGER = logging.getLogger(__name__)
@@ -110,6 +118,7 @@ class ProxyListenerDynamoDB(ProxyListener):
             return error_response_throughput()
 
         ProxyListenerDynamoDB.thread_local.existing_item = None
+        table_def = None
         if "TableName" in data:
             table_def = DynamoDBRegion.get().table_definitions.get(data["TableName"]) or {}
 
@@ -461,24 +470,41 @@ class ProxyListenerDynamoDB(ProxyListener):
                     record["dynamodb"]["StreamViewType"] = stream_spec["StreamViewType"]
 
         elif action == "CreateTable":
-            if "StreamSpecification" in data:
-                if response.status_code == 200:
-                    content = json.loads(to_str(response._content))
+            if response.status_code == 200:
+
+                table_definitions = (
+                    DynamoDBRegion.get().table_definitions.get(data["TableName"]) or {}
+                )
+                if "TableId" not in table_definitions:
+                    table_definitions["TableId"] = long_uid()
+
+                if "SSESpecification" in table_definitions:
+                    sse_specification = table_definitions.pop("SSESpecification")
+                    table_definitions["SSEDescription"] = get_sse_description(sse_specification)
+
+                content = json.loads(to_str(response.content))
+                if table_definitions:
+                    table_content = content.get("Table", {})
+                    table_content.update(table_definitions)
+                    content["TableDescription"].update(table_content)
+                    update_response_content(response, content)
+
+                if "StreamSpecification" in data:
                     create_dynamodb_stream(
                         data, content["TableDescription"].get("LatestStreamLabel")
                     )
 
+                if data.get("Tags"):
+                    table_arn = content["TableDescription"]["TableArn"]
+                    DynamoDBRegion.TABLE_TAGS[table_arn] = {
+                        tag["Key"]: tag["Value"] for tag in data["Tags"]
+                    }
+
             event_publisher.fire_event(
                 event_publisher.EVENT_DYNAMODB_CREATE_TABLE,
                 payload={"n": event_publisher.get_hash(table_name)},
             )
 
-            if data.get("Tags") and response.status_code == 200:
-                table_arn = json.loads(response._content)["TableDescription"]["TableArn"]
-                DynamoDBRegion.TABLE_TAGS[table_arn] = {
-                    tag["Key"]: tag["Value"] for tag in data["Tags"]
-                }
-
             return
 
         elif action == "DeleteTable":
@@ -533,11 +559,21 @@ class ProxyListenerDynamoDB(ProxyListener):
         elif action == "DescribeTable":
             table_name = data.get("TableName")
             table_props = DynamoDBRegion.get().table_properties.get(table_name)
+
             if table_props:
                 content = json.loads(to_str(response.content))
                 content.get("Table", {}).update(table_props)
                 update_response_content(response, content)
 
+            # Update only TableId and SSEDescription if present
+            table_definitions = DynamoDBRegion.get().table_definitions.get(table_name)
+            if table_definitions:
+                for key in ["TableId", "SSEDescription"]:
+                    if table_definitions.get(key):
+                        content = json.loads(to_str(response.content))
+                        content.get("Table", {})[key] = table_definitions[key]
+                        update_response_content(response, content)
+
         elif action == "TagResource":
             table_arn = data["ResourceArn"]
             table_tags = DynamoDBRegion.TABLE_TAGS
@@ -748,6 +784,14 @@ class ProxyListenerDynamoDB(ProxyListener):
             return default
 
 
+def get_sse_description(data):
+    return {
+        "Status": "ENABLED" if data["Enabled"] else "UPDATING",
+        "SSEType": data["SSEType"],
+        "KMSMasterKeyArn": aws_stack.kms_key_arn(data["KMSMasterKeyId"]),
+    }
+
+
 def handle_special_request(method, path, data, headers):
     if path.startswith("/shell") or method == "GET":
         if path == "/shell":
