@@ -14,9 +14,11 @@ from localstack import config
 from localstack.constants import (
     HEADER_LOCALSTACK_EDGE_URL,
     HEADER_LOCALSTACK_REQUEST_URL,
+    INTERNAL_AWS_ACCESS_KEY_ID,
     LOCALHOST,
     LOCALHOST_IP,
     LOCALSTACK_ROOT_FOLDER,
+    LS_LOG_TRACE_INTERNAL,
     PATH_USER_REQUEST,
 )
 from localstack.dashboard import infra as dashboard_infra
@@ -96,7 +98,7 @@ class ProxyListenerEdge(ProxyListener):
         # extract API details
         api, port, path, host = get_api_from_headers(headers, method=method, path=path, data=data)
 
-        if api and config.LS_LOG:
+        if api and is_trace_logging_enabled(headers):
             # print request trace for debugging, if enabled
             LOG.debug(
                 'IN(%s): "%s %s" - headers: %s - data: %s'
@@ -116,7 +118,7 @@ class ProxyListenerEdge(ProxyListener):
 
         if not port:
             if method == "OPTIONS":
-                if api and config.LS_LOG:
+                if api and is_trace_logging_enabled(headers):
                     # print request trace for debugging, if enabled
                     LOG.debug('OUT(%s): "%s %s" - status: %s' % (api, method, path, 200))
                 return 200
@@ -167,7 +169,7 @@ class ProxyListenerEdge(ProxyListener):
     def return_response(self, method, path, data, headers, response, request_handler=None):
         api = headers.get(HEADER_TARGET_API) or ""
 
-        if config.LS_LOG:
+        if is_trace_logging_enabled(headers):
             # print response trace for debugging, if enabled
             if api and api != API_UNKNOWN:
                 LOG.debug(
@@ -288,7 +290,7 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
     host = headers.get("host", "")
     auth_header = headers.get("authorization", "")
 
-    if not auth_header and "." not in host:
+    if not auth_header and not target and "." not in host:
         return result[0], result[1], path, host
 
     path = path or "/"
@@ -511,6 +513,15 @@ def get_service_port_for_account(service, headers):
 PROXY_LISTENER_EDGE = ProxyListenerEdge()
 
 
+def is_trace_logging_enabled(headers):
+    if not config.LS_LOG:
+        return False
+    if config.LS_LOG == LS_LOG_TRACE_INTERNAL:
+        return True
+    auth_header = headers.get("Authorization") or ""
+    return INTERNAL_AWS_ACCESS_KEY_ID not in auth_header
+
+
 def do_start_edge(bind_address, port, use_ssl, asynchronous=False):
     start_dns_server(asynchronous=True)
 
@@ -555,8 +566,12 @@ def start_component(component: str, port=None, use_ssl=True, asynchronous=False)
 def start_dns_server(asynchronous=False):
     try:
         # start local DNS server, if present
+        from localstack_ext import config as config_ext
         from localstack_ext.services import dns_server
 
+        if config_ext.DNS_ADDRESS in config.FALSE_STRINGS:
+            return
+
         if is_port_open(PORT_DNS):
             return
 
@@ -614,10 +629,11 @@ def run_process_as_sudo(component, port, asynchronous=False):
     sudo_cmd = "sudo -n "
     python_cmd = sys.executable
     edge_url = config.get_edge_url()
-    cmd = "%sPYTHONPATH=.:%s EDGE_FORWARD_URL=%s %s %s %s %s" % (
+    cmd = "%sPYTHONPATH=.:%s EDGE_FORWARD_URL=%s EDGE_BIND_HOST=%s %s %s %s %s" % (
         sudo_cmd,
         LOCALSTACK_ROOT_FOLDER,
         edge_url,
+        config.EDGE_BIND_HOST,
         python_cmd,
         __file__,
         component,
