@@ -18,10 +18,8 @@ from localstack.constants import FALSE_STRINGS, S3_STATIC_WEBSITE_HOSTNAME, TEST
 from localstack.services.cloudformation.deployment_utils import (
     PLACEHOLDER_AWS_NO_VALUE,
     PLACEHOLDER_RESOURCE_NAME,
-    dump_json_params,
-    param_defaults,
+    is_none_or_empty_value,
     remove_none_values,
-    select_parameters,
 )
 from localstack.services.cloudformation.service_models import (
     KEY_RESOURCE_STATE,
@@ -43,7 +41,6 @@ from localstack.utils.common import (
     to_bytes,
     to_str,
 )
-from localstack.utils.testutil import delete_all_s3_objects
 
 from localstack.services.cloudformation.models import *  # noqa: F401, isort:skip
 
@@ -83,243 +80,8 @@ def lambda_get_params():
     return lambda params, **kwargs: params
 
 
-def rename_params(func, rename_map):
-    def do_rename(params, **kwargs):
-        values = func(params, **kwargs) if func else params
-        for old_param, new_param in rename_map.items():
-            values[new_param] = values.pop(old_param, None)
-        return values
-
-    return do_rename
-
-
-def es_add_tags_params(params, **kwargs):
-    es_arn = aws_stack.es_domain_arn(params.get("DomainName"))
-    tags = params.get("Tags", [])
-    return {"ARN": es_arn, "TagList": tags}
-
-
-def get_ddb_provisioned_throughput(params, **kwargs):
-    args = params.get("ProvisionedThroughput")
-    if args == PLACEHOLDER_AWS_NO_VALUE:
-        return {}
-    if args:
-        if isinstance(args["ReadCapacityUnits"], str):
-            args["ReadCapacityUnits"] = int(args["ReadCapacityUnits"])
-        if isinstance(args["WriteCapacityUnits"], str):
-            args["WriteCapacityUnits"] = int(args["WriteCapacityUnits"])
-    return args
-
-
-def get_ddb_global_sec_indexes(params, **kwargs):
-    args = params.get("GlobalSecondaryIndexes")
-    if args:
-        for index in args:
-            provisoned_throughput = index["ProvisionedThroughput"]
-            if isinstance(provisoned_throughput["ReadCapacityUnits"], str):
-                provisoned_throughput["ReadCapacityUnits"] = int(
-                    provisoned_throughput["ReadCapacityUnits"]
-                )
-            if isinstance(provisoned_throughput["WriteCapacityUnits"], str):
-                provisoned_throughput["WriteCapacityUnits"] = int(
-                    provisoned_throughput["WriteCapacityUnits"]
-                )
-    return args
-
-
-def get_ddb_kinesis_stream_specification(params, **kwargs):
-    args = params.get("KinesisStreamSpecification")
-    if args:
-        args["TableName"] = params["TableName"]
-    return args
-
-
 # maps resource types to functions and parameters for creation
-RESOURCE_TO_FUNCTION = {
-    "S3::BucketPolicy": {
-        "create": {
-            "function": "put_bucket_policy",
-            "parameters": rename_params(
-                dump_json_params(None, "PolicyDocument"), {"PolicyDocument": "Policy"}
-            ),
-        }
-    },
-    "KinesisFirehose::DeliveryStream": {
-        "create": {
-            "function": "create_delivery_stream",
-            "parameters": select_parameters(
-                "DeliveryStreamName",
-                "DeliveryStreamType",
-                "S3DestinationConfiguration",
-                "ElasticsearchDestinationConfiguration",
-            ),
-        },
-        "delete": {
-            "function": "delete_delivery_stream",
-            "parameters": {"DeliveryStreamName": "DeliveryStreamName"},
-        },
-    },
-    "Elasticsearch::Domain": {
-        "create": [
-            {
-                "function": "create_elasticsearch_domain",
-                "parameters": select_parameters(
-                    "AccessPolicies",
-                    "AdvancedOptions",
-                    "CognitoOptions",
-                    "DomainName",
-                    "EBSOptions",
-                    "ElasticsearchClusterConfig",
-                    "ElasticsearchVersion",
-                    "EncryptionAtRestOptions",
-                    "LogPublishingOptions",
-                    "NodeToNodeEncryptionOptions",
-                    "SnapshotOptions",
-                    "VPCOptions",
-                ),
-            },
-            {"function": "add_tags", "parameters": es_add_tags_params},
-        ],
-        "delete": {
-            "function": "delete_elasticsearch_domain",
-            "parameters": {"DomainName": "DomainName"},
-        },
-    },
-    "Lambda::Version": {
-        "create": {
-            "function": "publish_version",
-            "parameters": select_parameters("FunctionName", "CodeSha256", "Description"),
-        }
-    },
-    "Lambda::EventSourceMapping": {
-        "create": {
-            "function": "create_event_source_mapping",
-            "parameters": select_parameters(
-                "FunctionName",
-                "EventSourceArn",
-                "Enabled",
-                "StartingPosition",
-                "BatchSize",
-                "StartingPositionTimestamp",
-            ),
-        }
-    },
-    "DynamoDB::Table": {
-        "create": [
-            {
-                "function": "create_table",
-                "parameters": {
-                    "TableName": "TableName",
-                    "AttributeDefinitions": "AttributeDefinitions",
-                    "KeySchema": "KeySchema",
-                    "ProvisionedThroughput": get_ddb_provisioned_throughput,
-                    "LocalSecondaryIndexes": "LocalSecondaryIndexes",
-                    "GlobalSecondaryIndexes": get_ddb_global_sec_indexes,
-                    "StreamSpecification": lambda params, **kwargs: (
-                        common.merge_dicts(
-                            params.get("StreamSpecification"),
-                            {"StreamEnabled": True},
-                            default=None,
-                        )
-                    ),
-                },
-                "defaults": {
-                    "ProvisionedThroughput": {
-                        "ReadCapacityUnits": 5,
-                        "WriteCapacityUnits": 5,
-                    }
-                },
-            },
-            {
-                "function": "enable_kinesis_streaming_destination",
-                "parameters": get_ddb_kinesis_stream_specification,
-            },
-        ],
-        "delete": {
-            "function": "delete_table",
-            "parameters": {"TableName": "TableName"},
-        },
-    },
-    "IAM::Role": {
-        "create": {
-            "function": "create_role",
-            "parameters": param_defaults(
-                dump_json_params(
-                    select_parameters(
-                        "Path",
-                        "RoleName",
-                        "AssumeRolePolicyDocument",
-                        "Description",
-                        "MaxSessionDuration",
-                        "PermissionsBoundary",
-                        "Tags",
-                    ),
-                    "AssumeRolePolicyDocument",
-                ),
-                {"RoleName": PLACEHOLDER_RESOURCE_NAME},
-            ),
-        },
-        "delete": {"function": "delete_role", "parameters": {"RoleName": "RoleName"}},
-    },
-    "ApiGateway::Account": {},
-    "ApiGateway::Model": {
-        "create": {
-            "function": "create_model",
-            "parameters": {
-                "name": "Name",
-                "restApiId": "RestApiId",
-            },
-            "defaults": {"contentType": "application/json"},
-        }
-    },
-    "ApiGateway::Deployment": {
-        "create": {
-            "function": "create_deployment",
-            "parameters": {
-                "restApiId": "RestApiId",
-                "stageName": "StageName",
-                "stageDescription": "StageDescription",
-                "description": "Description",
-            },
-        }
-    },
-    "ApiGateway::GatewayResponse": {
-        "create": {
-            "function": "put_gateway_response",
-            "parameters": {
-                "restApiId": "RestApiId",
-                "responseType": "ResponseType",
-                "statusCode": "StatusCode",
-                "responseParameters": "ResponseParameters",
-                "responseTemplates": "ResponseTemplates",
-            },
-        }
-    },
-    "StepFunctions::StateMachine": {
-        "create": {
-            "function": "create_state_machine",
-            "parameters": {
-                "name": ["StateMachineName", PLACEHOLDER_RESOURCE_NAME],
-                "definition": "DefinitionString",
-                "roleArn": lambda params, **kwargs: get_role_arn(params.get("RoleArn"), **kwargs),
-            },
-        },
-        "delete": {
-            "function": "delete_state_machine",
-            "parameters": {"stateMachineArn": "PhysicalResourceId"},
-        },
-    },
-    "StepFunctions::Activity": {
-        "create": {
-            "function": "create_activity",
-            "parameters": {"name": ["Name", PLACEHOLDER_RESOURCE_NAME], "tags": "Tags"},
-        },
-        "delete": {
-            "function": "delete_activity",
-            "parameters": {"activityArn": "PhysicalResourceId"},
-        },
-    },
-}
+RESOURCE_TO_FUNCTION = {}
 
 
 # ----------------
@@ -336,17 +98,6 @@ def get_secret_arn(secret_name, account_id=None):
     return storage.get(key) or storage.get(secret_name)
 
 
-def retrieve_topic_arn(topic_name):
-    topics = aws_stack.connect_to_service("sns").list_topics()["Topics"]
-    topic_arns = [t["TopicArn"] for t in topics if t["TopicArn"].endswith(":%s" % topic_name)]
-    return topic_arns[0]
-
-
-def get_role_arn(role_arn, **kwargs):
-    role_arn = resolve_refs_recursively(kwargs.get("stack_name"), role_arn, kwargs.get("resources"))
-    return aws_stack.role_arn(role_arn)
-
-
 def find_stack(stack_name):
     from localstack.services.cloudformation.cloudformation_api import find_stack as api_find_stack
 
@@ -458,11 +209,9 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             % (resource_id, resource, resources)
         )
     try:
-        # try to look up resource class
-        canonical_type = canonical_resource_type(resource_type)
-        resource_class = RESOURCE_MODELS.get(canonical_type)
-        if resource_class:
-            instance = resource_class(resource)
+        # convert resource props to resource entity
+        instance = get_resource_model_instance(resource_id, resources)
+        if instance:
             state = instance.fetch_and_update_state(stack_name=stack_name, resources=resources)
             return state
 
@@ -530,7 +279,7 @@ def extract_resource_attribute(
         resource = resources[resource_id]
 
     if not resource_state:
-        resource_state = retrieve_resource_details(resource_id, {}, resources, stack_name) or {}
+        resource_state = retrieve_resource_details(resource_id, {}, resources, stack_name)
         if not resource_state:
             raise DependencyNotYetSatisfied(
                 resource_ids=resource_id,
@@ -557,6 +306,8 @@ def extract_resource_attribute(
         )
 
     # extract resource specific attributes
+    # TODO: remove the code below - move into resource model classes!
+
     resource_props = resource.get("Properties", {})
     if resource_type == "Parameter":
         result = None
@@ -673,8 +424,7 @@ def canonical_resource_type(resource_type):
 
 def get_attr_from_model_instance(resource, attribute, resource_type, resource_id=None):
     resource_type = canonical_resource_type(resource_type)
-    # TODO: remove moto.MODEL_MAP here
-    model_class = RESOURCE_MODELS.get(resource_type) or parsing.MODEL_MAP.get(resource_type)
+    model_class = RESOURCE_MODELS.get(resource_type)
     if not model_class:
         if resource_type not in ["AWS::Parameter", "Parameter"]:
             LOG.debug('Unable to find model class for resource type "%s"' % resource_type)
@@ -708,7 +458,9 @@ def resolve_ref(stack_name, ref, resources, attribute):
 
     is_ref_attribute = attribute in ["Ref", "PhysicalResourceId", "Arn"]
     if is_ref_attribute:
-        resolve_refs_recursively(stack_name, resources.get(ref, {}), resources)
+        # extract the Properties here, as we only want to recurse over the resource props...
+        resource_props = resources.get(ref, {}).get("Properties")
+        resolve_refs_recursively(stack_name, resource_props, resources)
         return determine_resource_physical_id(
             resource_id=ref,
             resources=resources,
@@ -749,7 +501,7 @@ def resolve_ref(stack_name, ref, resources, attribute):
 
 
 # Using a @prevent_stack_overflow decorator here to avoid infinite recursion
-# in case we load stack exports that have circula dependencies (see issue 3438)
+# in case we load stack exports that have circular dependencies (see issue 3438)
 # TODO: Potentially think about a better approach in the future
 @prevent_stack_overflow(match_parameters=True)
 def resolve_refs_recursively(stack_name, value, resources):
@@ -959,15 +711,25 @@ def update_resource(resource_id, resources, stack_name):
         return
     LOG.info("Updating resource %s of type %s" % (resource_id, resource_type))
 
-    canonical_type = canonical_resource_type(resource_type)
-    resource_class = RESOURCE_MODELS.get(canonical_type)
-    if resource_class:
-        instance = resource_class(resource)
+    instance = get_resource_model_instance(resource_id, resources)
+    if instance:
         result = instance.update_resource(resource, stack_name=stack_name, resources=resources)
         instance.fetch_and_update_state(stack_name=stack_name, resources=resources)
         return result
 
 
+def get_resource_model_instance(resource_id, resources) -> GenericBaseModel:
+    """Obtain a typed resource entity instance representing the given stack resource."""
+    resource = resources[resource_id]
+    resource_type = get_resource_type(resource)
+    canonical_type = canonical_resource_type(resource_type)
+    resource_class = RESOURCE_MODELS.get(canonical_type)
+    if not resource_class:
+        return None
+    instance = resource_class(resource)
+    return instance
+
+
 def fix_account_id_in_arns(params):
     def fix_ids(o, **kwargs):
         if isinstance(o, dict):
@@ -1016,7 +778,8 @@ def prepare_template_body(req_data):
 
 
 def deploy_resource(resource_id, resources, stack_name):
-    return execute_resource_action(resource_id, resources, stack_name, ACTION_CREATE)
+    result = execute_resource_action(resource_id, resources, stack_name, ACTION_CREATE)
+    return result
 
 
 def delete_resource(resource_id, resources, stack_name):
@@ -1260,9 +1023,6 @@ def configure_resource_via_sdk(
     # remove None values, as they usually raise boto3 errors
     params = remove_none_values(params)
 
-    # run pre-actions
-    run_pre_create_actions(action_name, resource_id, resources, resource_type, stack_name, params)
-
     # convert boolean strings
     #  (TODO: we should find a more reliable mechanism than this opportunistic/probabilistic approach!)
     params_before_conversion = copy.deepcopy(params)
@@ -1292,160 +1052,13 @@ def configure_resource_via_sdk(
         )
         raise e
 
-    # run post-actions
-    run_post_create_actions(action_name, resource_id, resources, resource_type, stack_name, result)
-
     return result
 
 
-# TODO: move as individual functions to RESOURCE_TO_FUNCTION
-def run_pre_create_actions(
-    action_name, resource_id, resources, resource_type, stack_name, resource_params
-):
-    resource = resources[resource_id]
-    resource_props = resource["Properties"] = resource.get("Properties", {})
-    if resource_type == "IAM::Role" and action_name == ACTION_DELETE:
-        iam = aws_stack.connect_to_service("iam")
-        role_name = resource_props["RoleName"]
-        for policy in iam.list_attached_role_policies(RoleName=role_name).get(
-            "AttachedPolicies", []
-        ):
-            iam.detach_role_policy(RoleName=role_name, PolicyArn=policy["PolicyArn"])
-    if resource_type == "S3::Bucket" and action_name == ACTION_DELETE:
-        s3 = aws_stack.connect_to_service("s3")
-        bucket_name = resource_props.get("BucketName")
-        try:
-            s3.delete_bucket_policy(Bucket=bucket_name)
-        except Exception:
-            pass
-        # TODO: verify whether AWS CF automatically deletes all bucket objects, or fails if bucket is non-empty
-        try:
-            delete_all_s3_objects(bucket_name)
-        except Exception as e:
-            if "NoSuchBucket" not in str(e):
-                raise
-        # hack: make sure the bucket actually exists, to prevent delete_bucket operation later on from failing
-        s3.create_bucket(Bucket=bucket_name)
-
-
-# TODO: move as individual functions to RESOURCE_TO_FUNCTION
-def run_post_create_actions(action_name, resource_id, resources, resource_type, stack_name, result):
-    if action_name == ACTION_DELETE:
-        return result
-
-    resource = resources[resource_id]
-    resource_props = resource["Properties"] = resource.get("Properties", {})
-
-    # some resources have attached/nested resources which we need to create recursively now
-    if resource_type == "ApiGateway::RestApi":
-        body = resource_props.get("Body")
-        if body:
-            client = aws_stack.connect_to_service("apigateway")
-            body = json.dumps(body) if isinstance(body, dict) else body
-            client.put_rest_api(restApiId=result["id"], body=to_bytes(body))
-
-    elif resource_type == "SNS::Topic":
-        subscriptions = resource_props.get("Subscription", [])
-        for subscription in subscriptions:
-            if is_none_or_empty_value(subscription):
-                continue
-            endpoint = resolve_refs_recursively(stack_name, subscription["Endpoint"], resources)
-            topic_arn = retrieve_topic_arn(resource_props["TopicName"])
-            aws_stack.connect_to_service("sns").subscribe(
-                TopicArn=topic_arn, Protocol=subscription["Protocol"], Endpoint=endpoint
-            )
-
-    elif resource_type == "S3::Bucket":
-        tags = resource_props.get("Tags")
-        if tags:
-            aws_stack.connect_to_service("s3").put_bucket_tagging(
-                Bucket=resource_props["BucketName"], Tagging={"TagSet": tags}
-            )
-
-    elif resource_type == "IAM::Role":
-        policies = resource_props.get("Policies", [])
-        for policy in policies:
-            policy = policy[0] if isinstance(policy, list) and len(policy) == 1 else policy
-            iam = aws_stack.connect_to_service("iam")
-            if policy == PLACEHOLDER_AWS_NO_VALUE:
-                continue
-            if not isinstance(policy, dict):
-                LOG.info(
-                    'Invalid format of policy for IAM role "%s": %s'
-                    % (resource_props.get("RoleName"), policy)
-                )
-                continue
-            pol_name = policy.get("PolicyName")
-            doc = dict(policy["PolicyDocument"])
-            doc["Version"] = doc.get("Version") or IAM_POLICY_VERSION
-            statements = (
-                doc["Statement"] if isinstance(doc["Statement"], list) else [doc["Statement"]]
-            )
-            for statement in statements:
-                if isinstance(statement.get("Resource"), list):
-                    # filter out empty resource strings
-                    statement["Resource"] = [r for r in statement["Resource"] if r]
-            doc = json.dumps(doc)
-            LOG.debug(
-                "Running put_role_policy(...) for IAM::Role policy: %s %s %s"
-                % (resource_props["RoleName"], pol_name, doc)
-            )
-            iam.put_role_policy(
-                RoleName=resource_props["RoleName"],
-                PolicyName=pol_name,
-                PolicyDocument=doc,
-            )
-
-    elif resource_type == "IAM::Policy":
-        # associate policies with users, groups, roles
-        groups = resource_props.get("Groups", [])
-        roles = resource_props.get("Roles", [])
-        users = resource_props.get("Users", [])
-        policy_arn = aws_stack.policy_arn(resource_props.get("PolicyName"))
-        iam = aws_stack.connect_to_service("iam")
-        for group in groups:
-            iam.attach_group_policy(GroupName=group, PolicyArn=policy_arn)
-        for role in roles:
-            iam.attach_role_policy(RoleName=role, PolicyArn=policy_arn)
-        for user in users:
-            iam.attach_user_policy(UserName=user, PolicyArn=policy_arn)
-
-    elif resource_type == "IAM::InstanceProfile":
-        if resource_props.get("Roles", []):
-            iam = aws_stack.connect_to_service("iam")
-            iam.add_role_to_instance_profile(
-                InstanceProfileName=resource_props["InstanceProfileName"],
-                RoleName=resource_props["Roles"][0],
-            )
-    elif resource_type == "IAM::User":
-        iam = aws_stack.connect_to_service("iam")
-        username = resource_props.get("UserName")
-        for group in resource_props.get("Groups", []):
-            iam.add_user_to_group(UserName=username, GroupName=group)
-        for managed_policy in resource_props.get("ManagedPolicyArns", []):
-            iam.attach_user_policy(UserName=username, PolicyArn=managed_policy)
-        for policy in resource_props.get("Policies", []):
-            policy_doc = json.dumps(policy.get("PolicyDocument"))
-            iam.put_user_policy(
-                UserName=username, PolicyName=policy.get("PolicyName"), PolicyDocument=policy_doc
-            )
-        login_profile = resource_props.get("LoginProfile")
-        if login_profile:
-            iam.create_login_profile(
-                UserName=username,
-                Password=login_profile.get("Password"),
-                PasswordResetRequired=login_profile.get("PasswordResetRequired"),
-            )
-
-
 def get_action_name_for_resource_change(res_change):
     return {"Add": "CREATE", "Remove": "DELETE", "Modify": "UPDATE"}.get(res_change)
 
 
-def is_none_or_empty_value(value):
-    return not value or value == PLACEHOLDER_AWS_NO_VALUE
-
-
 # TODO: this shouldn't be called for stack parameters
 def determine_resource_physical_id(
     resource_id, resources=None, stack=None, attribute=None, stack_name=None
@@ -1469,7 +1082,7 @@ def determine_resource_physical_id(
         if result:
             return result
 
-    # TODO: put logic into resource-specific model classes
+    # TODO: put logic into resource-specific model classes!
     if resource_type == "ApiGateway::RestApi":
         result = resource_props.get("id")
         if result:
@@ -1552,8 +1165,10 @@ def update_resource_details(stack, resource_id, details, action=None):
         resource["PhysicalResourceId"] = details["KeyMetadata"]["KeyId"]
 
     if resource_type == "EC2::Instance":
-        if action == "CREATE":
+        if details and isinstance(details, list) and hasattr(details[0], "id"):
             resource["PhysicalResourceId"] = details[0].id
+        if isinstance(details, dict) and details.get("InstanceId"):
+            resource["PhysicalResourceId"] = details["InstanceId"]
 
     if resource_type == "EC2::SecurityGroup":
         resource["PhysicalResourceId"] = details["GroupId"]
@@ -1627,9 +1242,6 @@ def add_default_resource_props(
     elif res_type == "AWS::SQS::Queue" and not props.get("QueueName"):
         props["QueueName"] = "queue-%s" % short_uid()
 
-    elif res_type == "AWS::SQS::QueuePolicy" and not resource.get("PhysicalResourceId"):
-        resource["PhysicalResourceId"] = _generate_res_name()
-
     elif res_type == "AWS::IAM::ManagedPolicy" and not resource.get("ManagedPolicyName"):
         resource["ManagedPolicyName"] = _generate_res_name()
 
@@ -1693,6 +1305,9 @@ def add_default_resource_props(
             # append tags, to allow us to determine in service_models.py whether this key is already deployed
             tags.append({"Key": "localstack-key-id", "Value": short_uid()})
 
+    elif res_type == "AWS::IAM::ManagedPolicy":
+        props["ManagedPolicyName"] = props.get("ManagedPolicyName") or _generate_res_name()
+
     # generate default names for certain resource types
     default_attrs = (("AWS::IAM::Role", "RoleName"), ("AWS::Events::Rule", "Name"))
     for entry in default_attrs:
@@ -1887,7 +1502,8 @@ class TemplateDeployer(object):
 
         physical_id = physical_id or determine_resource_physical_id(resource_id, stack=stack)
         if not resource.get("PhysicalResourceId") or action == "UPDATE":
-            resource["PhysicalResourceId"] = physical_id
+            if physical_id:
+                resource["PhysicalResourceId"] = physical_id
 
         # set resource status
         stack.set_resource_status(resource_id, "%s_COMPLETE" % action, physical_res_id=physical_id)
@@ -2197,12 +1813,14 @@ class TemplateDeployer(object):
         if not evaluate_resource_condition(resource, stack_name, new_resources):
             return
         # execute resource action
+        result = None
         if action == "Add":
             result = deploy_resource(resource_id, new_resources, stack_name)
         elif action == "Remove":
             result = delete_resource(resource_id, old_stack.resources, stack_name)
         elif action == "Modify":
             result = update_resource(resource_id, new_resources, stack_name)
+
         # update resource status and physical resource id
         stack_action = get_action_name_for_resource_change(action)
         self.update_resource_details(resource_id, result, stack=old_stack, action=stack_action)
