@@ -25,7 +25,7 @@ from contextlib import closing
 from datetime import date, datetime, timezone
 from multiprocessing.dummy import Pool
 from queue import Queue
-from typing import Callable, Dict, List, Optional, Sized, Type, Union
+from typing import Any, Callable, Dict, List, Optional, Sized, Type, Union
 from urllib.parse import parse_qs, urlparse
 
 import dns.resolver
@@ -490,39 +490,55 @@ def is_base64(s):
     return is_string(s) and re.match(regex, s)
 
 
-def md5(string):
+def md5(string: Union[str, bytes]) -> str:
     m = hashlib.md5()
     m.update(to_bytes(string))
     return m.hexdigest()
 
 
-def select_attributes(obj, attributes):
+def select_attributes(obj: Dict, attributes: List[str]) -> Dict:
     """Select a subset of attributes from the given dict (returns a copy)"""
     attributes = attributes if is_list_or_tuple(attributes) else [attributes]
     return dict([(k, v) for k, v in obj.items() if k in attributes])
 
 
-def remove_attributes(obj, attributes):
+def remove_attributes(obj: Dict, attributes: List[str], recursive: bool = False) -> Dict:
     """Remove a set of attributes from the given dict (in-place)"""
+    if recursive:
+
+        def _remove(o, **kwargs):
+            if isinstance(o, dict):
+                remove_attributes(o, attributes)
+            return o
+
+        return recurse_object(obj, _remove)
     attributes = attributes if is_list_or_tuple(attributes) else [attributes]
     for attr in attributes:
         obj.pop(attr, None)
     return obj
 
 
-def is_list_or_tuple(obj):
+def is_list_or_tuple(obj) -> bool:
     return isinstance(obj, (list, tuple))
 
 
-def in_docker():
+def ensure_list(obj: Any, wrap_none=False) -> List:
+    """Wrap the given object in a list, or return the object itself if it already is a list."""
+    if obj is None and not wrap_none:
+        return obj
+    return obj if isinstance(obj, list) else [obj]
+
+
+def in_docker() -> bool:
     return config.in_docker()
 
 
-def path_from_url(url):
+def path_from_url(url: str) -> str:
     return "/%s" % str(url).partition("://")[2].partition("/")[2] if "://" in url else url
 
 
-def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=["tcp"]):
+def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=None):
+    protocols = protocols or ["tcp"]
     port = port_or_url
     if is_number(port):
         port = int(port)
@@ -841,7 +857,7 @@ def chown_r(path, user):
             os.chown(os.path.join(root, filename), uid, gid)
 
 
-def chmod_r(path, mode):
+def chmod_r(path: str, mode: int):
     """Recursive chmod"""
     if not os.path.exists(path):
         return
@@ -853,7 +869,7 @@ def chmod_r(path, mode):
             os.chmod(os.path.join(root, filename), mode)
 
 
-def rm_rf(path):
+def rm_rf(path: str):
     """
     Recursively removes a file or directory
     """
@@ -878,7 +894,7 @@ def rm_rf(path):
         shutil.rmtree(path)
 
 
-def cp_r(src, dst, rm_dest_on_conflict=False, ignore_copystat_errors=False, **kwargs):
+def cp_r(src: str, dst: str, rm_dest_on_conflict=False, ignore_copystat_errors=False, **kwargs):
     """Recursively copies file/directory"""
     # attention: this patch is not threadsafe
     copystat_orig = shutil.copystat
@@ -927,7 +943,7 @@ def cp_r(src, dst, rm_dest_on_conflict=False, ignore_copystat_errors=False, **kw
         shutil.copystat = copystat_orig
 
 
-def disk_usage(path):
+def disk_usage(path: str):
     if not os.path.exists(path):
         return 0
 
@@ -959,7 +975,7 @@ def format_bytes(count, default="n/a"):
     return count
 
 
-def download(url, path, verify_ssl=True):
+def download(url: str, path: str, verify_ssl=True):
     """Downloads file at url to the given path"""
     # make sure we're creating a new session here to
     # enable parallel file downloads during installation!
@@ -1006,7 +1022,7 @@ def download(url, path, verify_ssl=True):
         s.close()
 
 
-def parse_request_data(method, path, data=None, headers={}):
+def parse_request_data(method, path, data=None, headers=None):
     """Extract request data either from query string (for GET) or request body (for POST)."""
     result = {}
     headers = headers or {}
@@ -1075,6 +1091,7 @@ def is_alpine():
     return CACHE["_is_alpine_"]
 
 
+# TODO: rename to "get_os()"
 def get_arch():
     if is_mac_os():
         return "osx"
@@ -1084,7 +1101,7 @@ def get_arch():
         return "linux"
     if is_windows():
         return "windows"
-    raise Exception("Unable to determine system architecture")
+    raise Exception("Unable to determine local operating system")
 
 
 def is_command_available(cmd):
@@ -1231,13 +1248,13 @@ def replace_in_file(search, replace, file_path):
         save_file(file_path, content_new)
 
 
-def to_str(obj, encoding=DEFAULT_ENCODING, errors="strict"):
+def to_str(obj: Union[str, bytes], encoding: str = DEFAULT_ENCODING, errors="strict") -> str:
     """If ``obj`` is an instance of ``binary_type``, return
     ``obj.decode(encoding, errors)``, otherwise return ``obj``"""
     return obj.decode(encoding, errors) if isinstance(obj, six.binary_type) else obj
 
 
-def to_bytes(obj, encoding=DEFAULT_ENCODING, errors="strict"):
+def to_bytes(obj: Union[str, bytes], encoding: str = DEFAULT_ENCODING, errors="strict") -> bytes:
     """If ``obj`` is an instance of ``text_type``, return
     ``obj.encode(encoding, errors)``, otherwise return ``obj``"""
     return obj.encode(encoding, errors) if isinstance(obj, six.text_type) else obj
@@ -1488,8 +1505,6 @@ def generate_ssl_cert(
         return cert_file_name, key_file_name
 
     if target_file and not overwrite and os.path.exists(target_file):
-        key_file_name = ""
-        cert_file_name = ""
         try:
             cert_file_name, key_file_name = store_cert_key_files(target_file)
         except Exception as e:
@@ -1582,13 +1597,14 @@ def generate_ssl_cert(
     return file_content
 
 
-def run_safe(_python_lambda, *args, **kwargs):
+def run_safe(_python_lambda, *args, _default=None, **kwargs):
     print_error = kwargs.get("print_error", False)
     try:
         return _python_lambda(*args, **kwargs)
     except Exception as e:
         if print_error:
             LOG.warning("Unable to execute function: %s" % e)
+        return _default
 
 
 def run_cmd_safe(**kwargs):
@@ -1648,7 +1664,8 @@ def do_run(cmd: str, run_cmd: Callable, cache_duration_secs: int):
     return result
 
 
-def run(cmd, cache_duration_secs=0, **kwargs):
+def run(cmd: Union[str, List[str]], cache_duration_secs=0, **kwargs):
+    # TODO: should be unified and replaced with safe_run(..) over time! (allowing only lists for cmd parameter)
     def run_cmd():
         return localstack.utils.run.run(cmd, **kwargs)
 
